package cc.smtweb.system.bpm.web.design.preview;

import cc.smtweb.framework.core.annotation.SwParam;
import cc.smtweb.framework.core.annotation.SwService;
import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwConsts;
import cc.smtweb.framework.core.common.SwEnum;
import cc.smtweb.framework.core.db.DbEngine;
import cc.smtweb.framework.core.db.vo.ModelCatalog;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.session.UserSession;
import cc.smtweb.framework.core.util.PubUtil;
import cc.smtweb.framework.core.util.StringUtil;
import cc.smtweb.system.bpm.util.TreeDataUtil;
import cc.smtweb.system.bpm.web.design.db.ModelCatalogCache;
import cc.smtweb.system.bpm.web.design.db.ModelProjectCache;
import cc.smtweb.system.bpm.web.design.form.ModelForm;
import cc.smtweb.system.bpm.web.design.form.ModelFormCache;
import cc.smtweb.system.bpm.web.sys.user.menu.MenuCache;
import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlan;
import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlanCache;
import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlanContent;
import cc.smtweb.system.bpm.web.sys.user.menuPlan.MenuPlanItem;
import org.apache.commons.lang3.StringUtils;

import java.util.*;

@SwService
public class PreviewMenuTreeService {
    @SwParam
    private DbEngine dbEngine;

    public R treeAll(@SwParam("module") String module, UserSession us) {
        String prj_id = ModelProjectCache.getInstance().getIdByModule(module);
        if(!SwConsts.SysParam.SYS_DEBUG){
            List<MenuVO> menuVOList = buildMenu(prj_id, module, us);
            if (PubUtil.isNotEmpty(menuVOList)) {
                return R.success(menuVOList);
            }
        }
        List<ModelForm> listForm;
        if (StringUtils.isNotEmpty(prj_id)) {
            listForm = new ArrayList<>(ModelFormCache.getInstance().getFormsByPrj(Long.parseLong(prj_id)));
        } else {
            listForm = new ArrayList<>(ModelFormCache.getInstance().getAll());
        }
        listForm.sort((o1, o2) -> StringUtil.chineseCompare(o1.getTitle(), o2.getTitle()));
        if (listForm.isEmpty()) throw new BizException("此项目无页面设计！");

        List<MenuVO> list = new ArrayList<>();
        Map<Long, MenuVO> map = new HashMap<>();
        for (ModelForm form : listForm) {
            if (SwEnum.FormType.PAGE.value != form.getType()) continue;
            List<Long> parentLog = ModelCatalogCache.getInstance().getAllParent(form.getMcId());
            for (Long parent_id : parentLog) {
                if (map.containsKey(parent_id)) continue;
                ModelCatalog catalog = ModelCatalogCache.getInstance().get(parent_id);
                if (catalog == null || catalog.isEmpty()) continue;
                MenuVO menu = setMenuParent(parent_id, map, list);
                if (menu == null) continue;
                setMenuChildren(map.get(menu.getParentId()), menu);
            }
            MenuVO menu = new MenuVO();
            menu.setId(form.getId());
            menu.setName(form.getTitle());
            menu.setPath("/bpm/" + module + "." + form.getName());
            menu.setParentId(form.getMcId());
            setMenuChildren(map.get(form.getMcId()), menu);
        }

        MenuVO root = new MenuVO();
        root.setName("项目");
        root.setPath(module);
        root.setId(-1L);
        List<MenuVO> data = TreeDataUtil.buildTree(root, list, MenuVO.createTreeHandler());

        return R.success(data);
    }

    public List<MenuVO> buildMenu(String prj_id, String module, UserSession us) {
        Set<MenuPlan> set = MenuPlanCache.getInstance().getByP(prj_id);
        if (PubUtil.isEmpty(set)) {
            return new ArrayList<>();
        }
        MenuPlan menuPlan = set.iterator().next();
        MenuPlanContent mpc = new MenuPlanContent(menuPlan.getContent());
        List<MenuVO> list = new ArrayList<>();
        for (MenuPlanItem mp : mpc.getChildren(0)) {
            List<MenuVO> menuVOList = buildMenu(-1, mp);
            if (menuVOList == null) continue;
            list.addAll(menuVOList);
        }
        MenuVO root = new MenuVO();
        root.setName("项目");
        root.setPath(module);
        root.setId(-1L);
        return TreeDataUtil.buildTree(root, list, MenuVO.createTreeHandler());
    }

    private List<MenuVO> buildMenu(long parent_id, MenuPlanItem mp) {
        if (mp == null) return null;
        List<MenuVO> list = new ArrayList<>();
        MenuVO menu = new MenuVO();
        menu.setId(mp.getId());
        menu.setName(mp.getLabel());
        menu.setIcon(mp.getIcon());
        menu.setPath(MenuCache.getInstance().getPath(mp.getMenu()));
        menu.setParentId(parent_id);
        List<MenuVO> child = new ArrayList<>();
        if (PubUtil.isNotEmpty(mp.getChildren())) {
            for (MenuPlanItem cmp : mp.getChildren()) {
                List<MenuVO> m = buildMenu(menu.getId(), cmp);
                if (m == null) continue;
                child.addAll(m);
            }
        }
        menu.setChildren(child);
        list.add(menu);
        return list;
    }

    private MenuVO setMenuParent(Long parent_id, Map<Long, MenuVO> map, List<MenuVO> list) {
        MenuVO parent = map.get(parent_id);
        if (parent != null) return parent;

        ModelCatalog parentBean = ModelCatalogCache.getInstance().get(parent_id);
        if (parentBean == null) return null;
        parent = map.get(parentBean.getId());
        if (parent != null) return parent;
        parent = new MenuVO();
        parent.setId(parentBean.getId());
        parent.setName(parentBean.getName());
        parent.setPath("");
        parent.setParentId(parentBean.getParentId());
        if (parentBean.getParentId() == -1) {
            list.add(parent);
        }
        map.put(parentBean.getId(), parent);
        setMenuParent(parentBean.getParentId(), map, list);
        return parent;
    }

    private void setMenuChildren(MenuVO parent, MenuVO child) {
        if (parent == null) return;
        List<MenuVO> childs = parent.getChildren();
        if (childs == null) {
            childs = new ArrayList<>();
            parent.setChildren(childs);
        }
        childs.add(child);
    }
}
