package cc.smtweb.system.bpm.web.engine.model.flow.listcard.single;

import cc.smtweb.framework.core.common.R;
import cc.smtweb.framework.core.common.SwMap;
import cc.smtweb.framework.core.db.impl.DefaultEntity;
import cc.smtweb.framework.core.exception.BizException;
import cc.smtweb.framework.core.mvc.service.SwListData;
import cc.smtweb.framework.core.util.DateUtil;
import cc.smtweb.system.bpm.web.design.form.define.PageDataset;
import cc.smtweb.system.bpm.web.engine.dynPage.DynRetBean;
import cc.smtweb.system.bpm.web.engine.flow.FlowHelper;
import cc.smtweb.system.bpm.web.engine.flow.FlowInstance;
import cc.smtweb.system.bpm.web.engine.flow.FlowListener;
import cc.smtweb.system.bpm.web.engine.model.listcard.single.LCSingleSaveHandler;

/**
 * Created by Akmm at 2022-08-26 16:47
 * 带流程的简单卡片之保存
 */
public class FlowSingleSaveHandler<T extends DefaultEntity> extends LCSingleSaveHandler<T> implements FlowListener {
    protected FlowInstance flowInstance;

    @Override
    protected T readBeanFromPage(PageDataset pageDataSet, SwMap pageData) {
        T bean = super.readBeanFromPage(pageDataSet, pageData);

        long id = params.readLong("id");
        long taskId = params.readLong("taskId");

        SwMap data = pageData.readMap(FlowHelper.DATASET_NAME_FLOW);
        if (data == null) throw new BizException("没有找到待保存的流程数据！");
        data = data.readMap("form");
        if (data == null) throw new BizException("没有找到待保存的流程数据！");

        if (bean.isNew()) {
            flowInstance = FlowInstance.createByBillType(us, this, form.getBillType());
        } else {
            flowInstance = FlowInstance.createById(us,this, id, taskId);
        }
        flowInstance.readFromPage(data);
        flowInstance.getProcInst().setIsNew(bean.isNew());
        if (bean.isNew()) {
            flowInstance.getProcInst().setBillCode(getBillCode());
            flowInstance.getProcInst().setUrl(getBillUrl());
        }
        flowInstance.getProcInst().setBillInfo(getBillInfo());
        return bean;
    }

    @Override
    protected void saveBean(T bean) {
        super.saveBean(bean);
        flowInstance.save();
    }

    @Override
    protected void afterSave(SwMap ret, T bean) {
        super.afterSave(ret, bean);
        ret.put(FlowHelper.DATASET_NAME_FLOW, DynRetBean.createBean(flowInstance.getProcInst()));
        ret.put(FlowHelper.DATASET_NAME_COMMENT, DynRetBean.createList(SwListData.EMPTY));
        flowInstance.setPageRetExtra(ret);
    }

    //单据编号，新增保存时调用
    protected String getBillCode() {
        return flowInstance.getProcInst().getBillCode();
    }

    //单据信息，在待办中显示，每次保存均计算
    protected String getBillInfo() {
        return flowInstance.getProcInst().getBillCode() + "-" + DateUtil.nowDate();
    }

    //单据url，待办中跳链
    protected String getBillUrl() {
        return "";
    }
}
