package cc.smtweb.framework.core.session;

import cc.smtweb.framework.core.cache.redis.RedisManager;
import cc.smtweb.framework.core.mvc.controller.IEditor;
import cc.smtweb.framework.core.mvc.realm.exception.UnauthenticatedException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.context.request.RequestContextHolder;

import javax.servlet.http.HttpServletRequest;

/**
 * 〈session工具类〉
 *
 * @author kevin
 * @since 1.0.0
 */
public class SessionUtil {
  private SessionUtil() {
  }

  public static Object getSession() {
    return RequestContextHolder.currentRequestAttributes().getAttribute(IEditor.USER_SESSION, 0);
  }

  public static String readToken(HttpServletRequest request) {
    String token = request.getHeader("Auth-Token");

    if (token == null) {
      token = request.getParameter("auth_token");
    }
    return token;
  }

  public static UserSession checkSession(HttpServletRequest request, RedisManager redisManager) {
    String token = readToken(request);

    if (StringUtils.isBlank(token)) {
      throw new UnauthenticatedException("not find Auth-Token in header");
    }

    UserSession us = redisManager.get(token, UserSession.class);
    if (us == null) {
      throw new UnauthenticatedException("not find UserSession by token: " + token);
    }

    return us;
  }


  public static UserSession checkSession(String accessToken, RedisManager redisManager) {
    if (StringUtils.isBlank(accessToken)) {
      throw new UnauthenticatedException("not find Auth-Token in header");
    }

    UserSession us = redisManager.get(accessToken, UserSession.class);
    if (us == null) {
      throw new UnauthenticatedException("not find UserSession by token: " + accessToken);
    }

    return us;
  }

//    private static Session getShiroSession() {
//        Session session = SecurityUtils.getSubject().getSession();
//        if (session == null) {
//            throw new SmartException(ExceptionMessage.NO_LOGIN_ERROR);
//        }
//        return session;
//    }

//    public static String getSid() {
//        return getShiroSession().getId().toString();
//    }

//    public static UserSession getSession() {
//        Object session = getShiroSession().getAttribute(USER_SESSION);
//        if (session == null) {
//            throw new SmartException(ExceptionMessage.NO_LOGIN_ERROR);
//        }
//        UserSession userSession = (UserSession) session;
//        if (userSession.isChange()) {
//            throw new SmartException(ExceptionMessage.USER_PERMISSION_CHANGE);
//        }
//        return userSession;
//    }
//
//    public static void updateSession(UserSession userSession) {
//        getShiroSession().setAttribute(USER_SESSION, userSession);
//    }

//    public static String login(String userName, UserSession userSession) {
//        // 通过 shiro 登录验证
//        Subject subject = SecurityUtils.getSubject();
//        try {
//            // 统一默认密码登录
//            subject.login(new UsernamePasswordToken(userName, CommonUtil.INIT_PASSWORD));
//        } catch (ShiroException ex) {
//            throw new SmartException(ExceptionMessage.USER_ERROR);
//        }
//
//        Session session = subject.getSession();
//        if (session == null) {
//            throw new SmartException(ExceptionMessage.NO_LOGIN_ERROR);
//        }
//
//        session.setAttribute(USER_SESSION, userSession);
//        return session.getId().toString();
//    }
}
